package FeatureDataSource::OrigamiFeatureDataSource::OrigamiFeatureHandler;

use strict;
use SOAP::Lite;
#use SOAP::Lite +trace;


use FeatureDataSource;

my $ORGANISMSSVC='OrganismsSVC';

my $TARGETNS='urn:OrigamiTargetNS';
my $TYPESNS='urn:OrigamiTypeNS';

my $orgservice=undef;

BEGIN {
    my $mugen=$PreferencesHandler::Preferences->getDocumentElement();
    my @nodes=$mugen->getElementsByTagName('featuredatasources');
    my $rds=shift @nodes;
    my @nodes=$rds->getElementsByTagName('origami',0);
    if (scalar(@nodes)==0) {
	my $doctype=$PreferencesHandler::Preferences->getDoctype();
	$doctype->addElementDecl('origami','EMPTY');
	$doctype->addAttDef('origami','url','CDATA','#REQUIRED');
	my $origami=$PreferencesHandler::Preferences->createElement('origami');
	$origami->setAttribute('url','http://genome.jouy.inra.fr:8080/Origami');
	$rds->appendChild($origami);
    }
}

sub init {

    my $res=0;
    if (!$orgservice ) {
	my @nodes=$PreferencesHandler::Preferences->getElementsByTagName('origami');
	my $origami=shift @nodes;
	if ($origami) {
	    my $url=$origami->getAttribute('url');
#	    eval { 
		$orgservice=SOAP::Lite->service("$url/$ORGANISMSSVC?WSDL");
#	    };
	
	    return 0
		if (!$orgservice);


	    $orgservice->serializer(new AlphaSerializer());
	    $orgservice->maptype->{Organism}=$TYPESNS.$ORGANISMSSVC;
	    $orgservice->maptype->{ArrayOfOrganism}=$TYPESNS.$ORGANISMSSVC;
	    $orgservice->maptype->{Element}=$TYPESNS.$ORGANISMSSVC;
	    $orgservice->maptype->{ArrayOfElement}=$TYPESNS.$ORGANISMSSVC;
	    $orgservice->maptype->{Gene}=$TYPESNS.$ORGANISMSSVC;
	    $orgservice->maptype->{ArrayOfGene}=$TYPESNS.$ORGANISMSSVC;
	    $orgservice->maptype->{Feature}=$TYPESNS.$ORGANISMSSVC;
	    $orgservice->maptype->{ArrayOfFeature}=$TYPESNS.$ORGANISMSSVC;


	}
	
    }

    $res=1
	if ($orgservice);
	

    return $res;
}

sub getAllOrganisms {
    init unless $orgservice;
    my $organisms=$orgservice->getAllOrganisms();
    return $organisms;
} 

sub getOrganismWithId {
    my $id=shift;
    init unless $orgservice;
    return $orgservice->getOrganismWithId($id);
}

sub getElementsOfOrganisms {
    my $organisms=shift;
    init unless $orgservice;

    my @organismstab=();
   foreach my $org (@$organisms) {
       push @organismstab, new Organism($org);
    }
    my $SOAPorgtab=SOAP::Data->type('ArrayOfOrganism')->value(\@organismstab);
    return $orgservice->getElementsOfOrganisms($SOAPorgtab);
}

sub getElementWithId {
    my $id=shift;
    init unless $orgservice;
    return $orgservice->getElementWithId($id);
}

sub getSequenceOfElement {
    my $element=shift;

    init unless $orgservice;

    my $SOAPelement=new Element($element);
    return $orgservice->getSequenceOfElement($SOAPelement);
}


sub getFeaturesOfElement {
    my $element=shift;

    init unless $orgservice;

    my $SOAPelement=new Element($element);
    return $orgservice->getFeaturesOfElement($SOAPelement);
}

###Alphaserializer####################################################################
#####################################################################################
package AlphaSerializer;

@AlphaSerializer::ISA='SOAP::Serializer';


# Mthodes de srialization.
# Pour les instances des autres classes  srialiser, il suffit de recopier le contenu
# de cette mthode dans une mthode appele :
#              as_MachinChose
# Pour des classes dans des sous-packages (ex.: MonPackage::MaClasse), il faut remplacer
# les :: par des __, ce qui donnerait :
#              as_MonPackage__MaClasse
sub _alphamapper{
    my $self=shift;
    my ($value, $name, $type, $attr) = @_;

    return [
	    $name || $self->SUPER::gen_name(),
	    {%$attr, 'xsi:type' => $self->maptypetouri($type)},
	    [map {$self->encode_object($value->{$_}, $_)} sort { $a cmp $b } keys %$value ],
	    $self->gen_id($value)
	    ];
}
sub as_Organism {
    return (_alphamapper( @_ ) );
}

sub as_Element {
    return (_alphamapper( @_ ) );
   
}

sub as_Feature  {
    return (_alphamapper( @_ ) );

}

#######Empequatage SOAP#######################################################################
#############################################################################################
package Organism;

sub  new {
    my $class=shift;
    my $organism=shift;
   
    my $self={
	name=>SOAP::Data->type('string')->value($organism->{name}),
	id=>SOAP::Data->type('string')->value($organism->{id}),	
	shortName=>SOAP::Data->type('string')->value($organism->{shortName})
	};
    bless $self,$class;
    return $self ;
}

package Element;

sub new {
    my $class=shift;
    my $element=shift;
   
    my $self={
	accession=>SOAP::Data->type('string')->value($element->{accession}),
	description=>SOAP::Data->type('string')->value($element->{description}),
	id=>SOAP::Data->type('string')->value($element->{id}),
	organismId=>SOAP::Data->type('string')->value($element->{organismId}),
	size=>SOAP::Data->type('string')->value($element->{size}),
	type=>SOAP::Data->type('string')->value($element->{type})
	};
    return bless $self,$class;
}

package Feature;

sub new {
    my $class=shift;
    my $feature=shift;
    
    my $self={	
	type=>SOAP::Data->type('string')->value($feature->{type}),
	start=>SOAP::Data->type('int')->value($feature->{start}),
	stop=>SOAP::Data->type('int')->value($feature->{stop}),
	strand=>SOAP::Data->type('int')->value($feature->{strand})
    };
    return bless $self,$class;
}

1;


